package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ast.FunctionalExpression;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodVerifier;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

class BridgeCollector {
  MethodBinding[] bridges;
  
  MethodBinding method;
  
  char[] selector;
  
  LookupEnvironment environment;
  
  Scope scope;
  
  BridgeCollector(ReferenceBinding functionalType, MethodBinding method) {
    this.method = method;
    this.selector = method.selector;
    this.environment = paramFunctionalExpression.enclosingScope.environment();
    this.scope = (Scope)paramFunctionalExpression.enclosingScope;
    collectBridges(new ReferenceBinding[] { functionalType });
  }
  
  void collectBridges(ReferenceBinding[] interfaces) {
    int length = (interfaces == null) ? 0 : interfaces.length;
    for (int i = 0; i < length; i++) {
      ReferenceBinding superInterface = interfaces[i];
      if (superInterface != null) {
        MethodBinding[] methods = superInterface.getMethods(this.selector);
        for (int j = 0, count = (methods == null) ? 0 : methods.length; j < count; j++) {
          MethodBinding inheritedMethod = methods[j];
          if (inheritedMethod != null && this.method != inheritedMethod)
            if (!inheritedMethod.isStatic() && !inheritedMethod.redeclaresPublicObjectMethod(this.scope)) {
              inheritedMethod = MethodVerifier.computeSubstituteMethod(inheritedMethod, this.method, this.environment);
              if (inheritedMethod != null && MethodVerifier.isSubstituteParameterSubsignature(this.method, inheritedMethod, this.environment) && 
                MethodVerifier.areReturnTypesCompatible(this.method, inheritedMethod, this.environment)) {
                MethodBinding originalInherited = inheritedMethod.original();
                MethodBinding originalOverride = this.method.original();
                if (!originalOverride.areParameterErasuresEqual(originalInherited) || TypeBinding.notEquals(originalOverride.returnType.erasure(), originalInherited.returnType.erasure()))
                  add(originalInherited); 
              } 
            }  
        } 
        collectBridges(superInterface.superInterfaces());
      } 
    } 
  }
  
  void add(MethodBinding inheritedMethod) {
    if (this.bridges == null) {
      this.bridges = new MethodBinding[] { inheritedMethod };
      return;
    } 
    int length = this.bridges.length;
    for (int i = 0; i < length; i++) {
      if (this.bridges[i].areParameterErasuresEqual(inheritedMethod) && TypeBinding.equalsEquals((this.bridges[i]).returnType.erasure(), inheritedMethod.returnType.erasure()))
        return; 
    } 
    System.arraycopy(this.bridges, 0, this.bridges = new MethodBinding[length + 1], 0, length);
    this.bridges[length] = inheritedMethod;
  }
  
  MethodBinding[] getBridges() {
    return this.bridges;
  }
}
